@@ -1,12 +1,17 @@
-from typing import List, Tuple
+from abc import ABC
 from ase import Atoms
+from clease.montecarlo.base import MCStep
+from clease.datastructures.system_changes import SystemChanges
 
 
-class MCObserver:
-    """Base class for all MC observers."""
+class MCObserver(ABC):
+    """Base class for all MC observers.
+
+    Child observers should override the observe_step() method."""
+
     name = "GenericObserver"
 
-    def __call__(self, system_changes: List[Tuple[int, str, str]]):
+    def __call__(self, system_changes: SystemChanges) -> None:
         """
         Gets information about the system changes and can perform some action
 
@@ -16,14 +21,22 @@ class MCObserver:
             index 26 is swapped with an Al atom occupying the atomic index 12,
             system_change = [(26, Mg, Al), (12, Al, Mg)]
         """
-        pass
+        raise NotImplementedError("Observer does not implement __call__")
+
+    def observe_step(self, mc_step: MCStep) -> None:
+        """Observe on a MCStep object. Defaults to __call__(system_changes)
+        for compatibility reasons. Child classes overriding this function
+        should therefore not call the super() version.
+        """
+        # use the __call__ method for backwards compatibility
+        self(mc_step.last_move)
 
     def reset(self) -> None:
         """Reset all values of the MC observer"""
-        pass
 
     def get_averages(self) -> dict:
         """Return averages in the form of a dictionary."""
+        # pylint: disable=no-self-use
         return {}
 
     def calculate_from_scratch(self, atoms: Atoms) -> None:
@@ -31,7 +44,6 @@ class MCObserver:
         Method for calculating the tracked value from scratch
         (i.e. without using fast update methods)
         """
-        pass
 
     def interval_ok(self, interval: int) -> bool:
         """
@@ -41,4 +53,5 @@ class MCObserver:
         :param interval: Interval controlling how often a MC observer will be
             called.
         """
+        # pylint: disable=no-self-use, unused-argument
         return True
